/*
	Gnash Qt4 XEmbed Demo
	Written (2016) by Nutchanon Wetchasit <Nutchanon.Wetchasit@gmail.com>
	
	Released to public domain under Creative Commons Zero 1.0
	(https://creativecommons.org/publicdomain/zero/1.0/)
	
	Note: This code is a simple example, it does not handle all kind of error.
	The error that this demo does not handle is:
		- Gnash terminated with error status.
*/
#include <QObject>
#include <QMainWindow>
#include <QTimer>
#include <QProcess>
#include <QMessageBox>
#include <QCloseEvent>
#include "ui_gnash-xembed-qt4.h"

namespace Ui {
	class GnashXEmbedQT4;
}

class GnashXEmbedQT4: public QMainWindow
{
	Q_OBJECT
	
private:
	Ui::GnashXEmbedQT4 *ui;
	QProcess *process;
	
public:
	// Initialize GUI
	explicit GnashXEmbedQT4(QWidget *parent=NULL): QMainWindow(parent), ui(new Ui::GnashXEmbedQT4)
	{
		ui->setupUi(this);
		QTimer::singleShot(0,this,SLOT(packWindow()));
	}
	
	// Invoke Gnash
	void startGnash() {	
		QX11EmbedContainer *embed;
		QStringList arguments;
		
		// Obtain reference to QX11EmbedContainer
		embed=this->findChild<QX11EmbedContainer*>("embed");
	
		// Start Gnash
		process=new QProcess();
		connect(process,SIGNAL(error(QProcess::ProcessError)),SLOT(process_error(QProcess::ProcessError)));
		arguments << "--xid";
		arguments << QString::number(embed->winId());
		arguments << "hovercolorbars.swf";
		process->start(QString("qt4-gnash"),arguments);
	}
	
	// Handle window's close button, kill Gnash before exiting
	void closeEvent(QCloseEvent*) {
		process->kill();
		process->waitForFinished();
		qApp->quit();
	}
	
	// Destructor
	virtual ~GnashXEmbedQT4() {
		delete process;
		delete ui;
	}
	
private slots:
	// Resize window to fit content
	void packWindow() {
		resize(sizeHint());
	}
	
	// Handle Gnash invocation error
	void process_error(QProcess::ProcessError error) {
		if(error==QProcess::FailedToStart)
			QMessageBox::critical(this,QString("Error"),QString("Failed to start Gnash."));
	}
	
	// Handle Exit button click, kill Gnash before exiting
	void on_exitbutton_clicked() {
		process->kill();
		process->waitForFinished();
		qApp->quit();
	}	
};

#include "gnash-xembed-qt4.moc"

// Main program
int main(int argc,char *argv[]) {
	// Tell Qt to always use native window-based rendering (or QX11EmbedContainer won't work)
	QCoreApplication::setAttribute(Qt::AA_NativeWindows,1);
	
	// Initialize Qt
	QApplication app(argc,argv);
	
	// Construct and show the main window
	GnashXEmbedQT4 window;
	window.show();
	
	// Start Gnash
	window.startGnash();
	
	// Start GUI loop
	return app.exec();
}
